//==========================================================================;
//
//  THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
//  KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
//  IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
//  PURPOSE.
//
//  Copyright (c) 2009 Interlink Electronics, Inc.
//--------------------------------------------------------------------------;
//
using System;
using System.Threading;
using System.Diagnostics;
using System.Reflection;
using System.IO;
using System.Drawing;
using System.Collections;
using Interlink.ePad.ePadVision;
using Interlink.ePad.ePadCore;
using HANDLE = System.IntPtr;
using System.Windows.Forms;

/// <summary>
/// Main Business Logic class
/// </summary>
namespace ePad_VisionDemo
{
    public class ePadVisDemo
    {
        public ePadVision vis;
        public HANDLE m_hWidgetFeedback;
        public HANDLE m_hCloseFormEvent;
        public HANDLE m_DoneEvent;
        public bool m_WaitingForWidgetEvent;
        public volatile bool Status;
        public struct eventData {
            public byte m_wid;
            public EventType m_wtype;
            public EventCode m_ecode;
            public eventData(byte wid, EventType wtype, EventCode wcode ) {
                m_wid = wid;
                m_wtype = wtype;
                m_ecode = wcode;
            }
        };
        Queue eventQ = null;
        IntPtr[] handle;
        Guid myguid;
        const string msgCaption = "ePad-vision Compliance Text Demo";
        
        private string agreeText;
        private string compliText;
        private string affirmText;
        public string AgreeText
        {
            get
            {
                return agreeText;
            }
            set
            {
                agreeText = value;
            }
        }
        public string CompliText
        {
            get
            {
                return compliText;
            }
            set
            {
                compliText = value;
            }
        }
        public string AffirmText
        {
            get
            {
                return affirmText;
            }
            set
            {
                affirmText = value;
            }
        }

        public ePadVisDemo()
        {
            vis = new ePadVision();
            vis.SignatureDataEvent += new SignatureDataDelegate(SignatureDataHandler);
            vis.NotificationDataEvent += new EventDataDelegate(WidgetDataHandler);
            m_hWidgetFeedback = Win32.CreateEvent(HANDLE.Zero, false, false, null);
            m_hCloseFormEvent = Win32.CreateEvent(HANDLE.Zero, false, false, null);
            m_DoneEvent = Win32.CreateEvent(HANDLE.Zero, false, false, null);
            m_WaitingForWidgetEvent = false;
            Status = false;
            handle = new IntPtr[2];
            handle[1] = this.m_hWidgetFeedback;
            handle[0] = this.m_hCloseFormEvent;
            myguid = new Guid("90EE1718-3A66-4aeb-87D4-F44885CDF7B4");
        }
        private void SignatureDataHandler(Int32 xAxis, Int32 yAxis, Int32 pressure, Int32 timeStamp)
        {
            string line = string.Format("{0:x}, {1:x}, {2:x}, {3:x};\n", xAxis, yAxis, pressure, timeStamp);
             Trace.WriteLine(line);
            //Invoke(new AddPoint(AddSignaturePoint), xAxis, yAxis, pressure, timeStamp);
        }
        private void WidgetDataHandler(byte widgetID, EventType widgetType, EventCode eventCode, byte[] data)
        {
            string line;

            line = string.Format("Widget ID: {0}, Type: {1}, Event: {2};\n", widgetID, widgetType, eventCode);
            Trace.Write(line);
            eventData newEntry = new eventData(widgetID, widgetType, eventCode);
            
            if ( eventQ == null ) {
                eventQ = new Queue();
            } 
            eventQ.Enqueue(newEntry);
            Trace.WriteLine("waiting?" + m_WaitingForWidgetEvent + "queue count " + eventQ.Count);
            if (m_WaitingForWidgetEvent)
            {
                Win32.SetEvent(m_hWidgetFeedback);
                //m_WaitingForWidgetEvent = false;
            }
        }
        private int WaitForMultiEvents()
        {
            int ret = -1;
            do
            {
                ret = Win32.MsgWaitForMultipleObjects(2, handle, false, Win32.INFINITE, (uint)Win32.QueueStatusFlags.QS_ALLINPUT);
                if (ret >= 2)
                {
                    System.Windows.Forms.Application.DoEvents();
                }
                else
                    break;
            } while (true);
            //Trace.WriteLine("WaitForMultiEvents return " + ret);
            return ret;
        }
        public void Start()
        {
            bool ret = false;
            //main script routine
            if (vis.DeviceCount > 0)
            {
                try
                {
                    vis.Open(0);
                }
                catch (ePadException ex)
                {
                    MessageBox.Show("ePad-vision device open failed.", msgCaption, MessageBoxButtons.OK, MessageBoxIcon.Stop);
                }                    
                //disable the screen saver 
                vis.ScreenSaver.Stop();
                //disable default background image
                vis.SignaturePad.EnableBackgroundImage = false;
                Guid curguid = vis.SignaturePad.CustomizedBackgroundImageID;

                // get a reference to the current assembly if this is the first run
                // if yes, then download the default background from the resource
                if (curguid != myguid )
                {
                    byte[] imageData;
                    Assembly a = this.GetType().Assembly;
                    string[] resNames = a.GetManifestResourceNames();
                    foreach (string s in resNames)
                    {
                        // get a list of resource names from the manifest
                        if (s.EndsWith(".sign.bmp"))
                        {
                            Stream imgStream = a.GetManifestResourceStream(s);
                            if (null != imgStream)
                            {
                                //retrieve the bitmap from the resource via binaryread
                                BinaryReader br = new BinaryReader(imgStream);
                                imageData = br.ReadBytes((int)imgStream.Length);
                                br.Close();
                                LCDPixelRange lrange = new LCDPixelRange();
                                lrange.X = 0;
                                lrange.Y = 60;
                                lrange.Width = 480;
                                lrange.Height = 150;
                                
                                vis.SignaturePad.SetCustomizedBackgroundImage(myguid, lrange, imageData);
                                
                                imgStream.Close();
                            }
                        }
                    }
                }

                if (screen1())
                {
                    if (screen2())
                    {
                        if (screen3a())
                        {
                            int ret3 = screen3();
                            if (ret3 > 0)
                            {
                                if (screen4())
                                {
                                    screen5(1);
                                    ret = true;
                                }
                                else
                                {
                                    screen5(0);
                                }
                            }
                            else if (ret3 == 0)
                            {
                                screen5(0);
                            }
                        }
                        else
                        {
                            screen5(0);
                        }
                    }
                }
                else
                {
                    screen5(0);
                }
                vis.Close();
                Trace.WriteLine("ePad-vision closed");
                Win32.SetEvent(this.m_DoneEvent);

            }
            else
            {
                MessageBox.Show("No ePad-vision Device found, please plug-in ePad-vision device and try again", msgCaption, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            Status = ret;
            return;
        }
        // input byte widgetid
        // input eventcode expecting event code1
        // input eventcode expecting event code2
        // return 1 or 2 respectively
        public int WaitForWidgetEvent(byte wid, EventCode expectCode1, EventCode expectCode2)
        {
            //wait for the events
            int nOK = 0;
            do
            {
                int ret = WaitForMultiEvents();
                if (ret == 1)
                {
                    //consume all the entries in the queue
                    eventData eobj;
                    while (eventQ.Count > 0)
                    {
                        eobj = (eventData)eventQ.Dequeue();
                        if (eobj.m_wid == wid)
                        {
                            if (eobj.m_ecode == expectCode1)
                            {
                                nOK = 1;
                                Trace.WriteLine("waiting for " + wid + " got result " + nOK);
                                break;
                            }
                            else if (eobj.m_ecode == expectCode2)
                            {
                                nOK = 2;
                                Trace.WriteLine("waiting for " + wid + " got result " + nOK);
                                break;
                            }
                        }
                    }
                }
                else
                {
                    Trace.WriteLine("WaitForMultiEvents error !! returns " +  ret);
                }
                if (nOK > 0)
                {
                    Win32.ResetEvent(this.m_hWidgetFeedback);
                }
            } while (nOK == 0);
            return nOK;
        }

        // input byte widgetid
        // input eventcode expecting event code 
        // return true success.
        public bool WaitForWidgetEvent(byte wid, EventCode expectCode) 
        {
            //wait for the events
            bool bOK = false;
            do
            {
                int ret = WaitForMultiEvents();
                if (ret == 1)
                {
                    //consume all the entries in the queue
                    eventData eobj;
                    while (eventQ.Count > 0)
                    {
                        eobj = (eventData)eventQ.Dequeue();
                        Trace.WriteLine("WaitForWidgetEvent got " + eobj.m_wid + " and code = " + eobj.m_ecode);
                        if (eobj.m_wid == wid && eobj.m_ecode == expectCode)
                        {
                            Trace.WriteLine("WaitForWidgetEvent return true");
                            bOK = true;
                            break;
                        }
                    }
                }
                else
                {
                    Trace.WriteLine("!!!!!waitforMultiEvents return " + ret + " bOK = " + bOK);
                }
                if (bOK)
                {
                    Win32.ResetEvent(this.m_hWidgetFeedback);
                }
            } while (bOK == false);
            return bOK; 
        }
        #region screen3a
        private bool screen3a()
        {
            bool bret = false;
            //remove all the previous widgets
            m_WaitingForWidgetEvent = true;
            vis.Widgets.RemoveAllWidgets();
            int ret = WaitForMultiEvents();
            if (ret == 0)
            {
                return false;
            }
            while (eventQ.Count > 0)
            {
                eventQ.Dequeue();
            }
            Win32.ResetEvent(this.m_hWidgetFeedback);
            //make inking disable            
            InkRegion region = vis.SignaturePad.InkRegion;
            region.Enabled = false;
            
            vis.SignaturePad.InkRegion = region;
            LCDColor blue;
            blue.Red = 0;
            blue.Green = 0;
            blue.Blue = 255;
            blue.Alpha = 255;
            //set the bkcolor same with widget color
            vis.SignaturePad.BackgroundColor = blue;
            //make lcd background image off
            vis.SignaturePad.EnableBackgroundImage = false;
            //change font size 
            //create a full screen widget text box with text
            WidgetInfo wg1 = new WidgetInfo();

            LCDColor grey;
            grey.Red = 184;
            grey.Green = 181;
            grey.Blue = 187;
            grey.Alpha = 255;
            wg1.BgColor = grey;
            if (agreeText.Length > 940)
            {
                agreeText.Remove(940);
            }
            wg1.Text = this.agreeText;
            wg1.WidgetID = 1;
            wg1.WidgetType = WidgetType.TEXT;
            wg1.FontID = (byte)FontID.SAN_B_20;
            wg1.FgColor.Red = 0;
            wg1.FgColor.Green = 0;
            wg1.FgColor.Blue = 0;
            wg1.FgColor.Alpha = 255;
            wg1.Effect = WidgetEffect.THREED;
            wg1.Left = 1;
            wg1.Top = 1;
            wg1.Width = 479;
            wg1.Height = 208;
            this.m_WaitingForWidgetEvent = true;

            vis.Widgets.CreateWidget(wg1);
            int nRet = WaitForWidgetEvent(wg1.WidgetID, EventCode.CREATE_SUCCEED, EventCode.CREATE_FAILED);
            if (nRet == 2)
            {
                throw new ePadException(HERROR.F_INTERNAL_ERROR);
            }
            Trace.WriteLine("wg1 creating return " + nRet);           

            WidgetInfo bt1 = new WidgetInfo();
            WidgetInfo bt2 = new WidgetInfo();

            bt1.BgColor.Red = 0;
            bt1.BgColor.Green = 255;
            bt1.BgColor.Blue = 0;
            bt1.BgColor.Alpha = 255;
            bt1.Text = "Disagree";
            bt1.WidgetID = 3;
            bt1.WidgetType = WidgetType.BUTTON;
            bt1.FontID = (byte)FontID.SAN_B_20;
            bt1.FgColor.Red = 0;
            bt1.FgColor.Green = 0;
            bt1.FgColor.Blue = 0;
            bt1.FgColor.Alpha = 255;
            bt1.Effect = WidgetEffect.THREED;
            bt1.Left = 120;
            bt1.Top = 210;
            bt1.Width = 90;
            bt1.Height = 60;
            bt1.GroupID = 2;

            this.m_WaitingForWidgetEvent = true;
            vis.Widgets.CreateWidget(bt1);
            nRet = WaitForWidgetEvent(bt1.WidgetID, EventCode.CREATE_SUCCEED, EventCode.CREATE_FAILED);
            if (nRet == 2)
            {
                throw new ePadException(HERROR.F_INTERNAL_ERROR);
            }
            Trace.WriteLine("bt1 creating return " + nRet);

            bt2 = bt1;
            bt2.Text = "Agree";
            bt2.Left = 280;
            bt2.WidgetID = 4;

            this.m_WaitingForWidgetEvent = true;
            vis.Widgets.CreateWidget(bt2);
            nRet = WaitForWidgetEvent(bt2.WidgetID, EventCode.CREATE_SUCCEED, EventCode.CREATE_FAILED);
            if (nRet == 2)
            {
                throw new ePadException(HERROR.F_INTERNAL_ERROR);
            }
            Trace.WriteLine("bt2 creating return " + nRet);
            //wait for the events
            bool bClicked = false;
            do
            {
                this.m_WaitingForWidgetEvent = true;
                Trace.WriteLine("final waiting...");
                //need to handle application exit event!!!
                ret = WaitForMultiEvents();
                if (ret == 0)
                {
                    Trace.WriteLine("screen0 quit since " + ret);
                    return false;
                }
                while (eventQ.Count > 0)
                {
                    eventData eobj = (eventData)eventQ.Dequeue();

                    Trace.WriteLine("final got the event:" + eobj.m_wid + ",code =" + eobj.m_ecode);
                    if (eobj.m_wid == bt1.WidgetID && eobj.m_ecode == EventCode.CLICKED)
                    {
                        bClicked = true;
                        bret = false;
                        break;
                    }
                    else if (eobj.m_wid == bt2.WidgetID && eobj.m_ecode == EventCode.CLICKED)
                    {
                        bClicked = true;
                        bret = true;
                        break;
                    }
                } //end of while
                Win32.ResetEvent(this.m_hWidgetFeedback);
            } while (bClicked == false);
            return bret;
        }
        #endregion
        #region screen1
        private bool screen1()
        {
            string text = "In this transaction, you have the option to capture your signature electronically or opt-out. Please make your selection below.";

            string rbtext1 = "Continue with signing process electronically";
            string rbtext2 = "Opt-out";
            bool bret = false;

            //make inking disable            
            InkRegion region = vis.SignaturePad.InkRegion;
            region.Enabled = false;
            byte activerd = 0;
            vis.SignaturePad.InkRegion = region;
            LCDColor blue;
            blue.Red = 0;
            blue.Green = 0;
            blue.Blue = 255;
            blue.Alpha = 255;
            //set the bkcolor same with widget color
            vis.SignaturePad.BackgroundColor = blue;
            //make lcd background image off
            vis.SignaturePad.EnableBackgroundImage = false;
            //change font size 
            //create a full screen widget text box with text
            WidgetInfo wg1 = new WidgetInfo();
            wg1.BgColor = blue;
            wg1.Text = text;
            wg1.WidgetID = 1;
            wg1.WidgetType = WidgetType.TEXT;
            wg1.FontID = (byte)FontID.SAN_B_24;
            wg1.FgColor.Red = 255;
            wg1.FgColor.Green = 255;
            wg1.FgColor.Blue = 0;
            wg1.FgColor.Alpha = 255;
            wg1.Effect = WidgetEffect.NONE;
            wg1.Left = 10;
            wg1.Top = 20;
            wg1.Width = 478;
            wg1.Height = 80;
            this.m_WaitingForWidgetEvent = true;
            
            vis.Widgets.CreateWidget(wg1);
            Trace.WriteLine("screen1 creating widget id 1...");
            int nRet = WaitForWidgetEvent(wg1.WidgetID, EventCode.CREATE_SUCCEED, EventCode.CREATE_FAILED);
            if (nRet == 2)
            {
                throw new ePadException(HERROR.F_INTERNAL_ERROR);
            }
            Trace.WriteLine("screen1 wg1 creating return " + nRet);

            //create two radio buttons
            WidgetInfo rd1 = new WidgetInfo();
            rd1.BgColor = blue;
            rd1.Text = rbtext1;
            rd1.WidgetID = 2;
            rd1.WidgetType = WidgetType.RADIOBUTTON;
            rd1.FontID = (byte)FontID.MON_R_18;
            rd1.FgColor.Red = 255;
            rd1.FgColor.Green = 255;
            rd1.FgColor.Blue = 0;
            rd1.FgColor.Alpha = 255;
            rd1.Effect = WidgetEffect.NONE;
            rd1.Left = 50;
            rd1.Top = 120;
            rd1.Width = 380;
            rd1.Height = 25;
            rd1.GroupID = 1;
            this.m_WaitingForWidgetEvent = true;

            vis.Widgets.CreateWidget(rd1);
            nRet = WaitForWidgetEvent(rd1.WidgetID, EventCode.CREATE_SUCCEED, EventCode.CREATE_FAILED);
            if (nRet == 2)
            {
                throw new ePadException(HERROR.F_INTERNAL_ERROR);
            }
            Trace.WriteLine("rd1 creating return " + nRet + ", length =" + rbtext1.Length);
       
            activerd = rd1.WidgetID;
            WidgetInfo rd2 = new WidgetInfo();

            rd2.BgColor = blue;
            rd2.Text = rbtext2;
            rd2.WidgetID = 3;
            rd2.WidgetType = WidgetType.RADIOBUTTON;
            rd2.FontID = (byte)FontID.MON_R_18;
            rd2.FgColor.Red = 255;
            rd2.FgColor.Green = 255;
            rd2.FgColor.Blue = 0;
            rd2.FgColor.Alpha = 255;
            rd2.Effect = WidgetEffect.NONE;
            rd2.Left = 50;
            rd2.Top = 170;
            rd2.Width = 90;
            rd2.Height = 25;
            rd2.GroupID = 1;
            this.m_WaitingForWidgetEvent = true;
            vis.Widgets.CreateWidget(rd2);
                 
            //waiting for whether event1 or 2 in return
            nRet = WaitForWidgetEvent(rd2.WidgetID, EventCode.CREATE_SUCCEED, EventCode.CREATE_FAILED);
            if (nRet == 2)
            {
                throw new ePadException(HERROR.F_INTERNAL_ERROR);
            }
            Trace.WriteLine("rd2 creating return " + nRet + ", length =" + rbtext2.Length);
            
            WidgetInfo bt1 = new WidgetInfo();
            bt1.BgColor.Red = 0;
            bt1.BgColor.Green = 255;
            bt1.BgColor.Blue = 0;
            bt1.BgColor.Alpha = 255;
            bt1.Text = "OK";
            bt1.WidgetID = 4;
            bt1.WidgetType = WidgetType.BUTTON;
            bt1.FontID = (byte)FontID.SAN_B_20;
            bt1.FgColor.Red = 0;
            bt1.FgColor.Green = 0;
            bt1.FgColor.Blue = 0;
            bt1.FgColor.Alpha = 255;
            bt1.Effect = WidgetEffect.THREED;
            bt1.Left = 200;
            bt1.Top = 210;
            bt1.Width = 80;
            bt1.Height = 60;
            bt1.GroupID = 2;
            this.m_WaitingForWidgetEvent = true;
            vis.Widgets.CreateWidget(bt1);
            nRet = WaitForWidgetEvent(bt1.WidgetID, EventCode.CREATE_SUCCEED, EventCode.CREATE_FAILED);
            if (nRet == 2)
            {
                throw new ePadException(HERROR.F_INTERNAL_ERROR);
            }
            Trace.WriteLine("bt1 creating return " + nRet);
            vis.SignaturePad.Refresh();

            //wait for the events
            bool bClicked = false;
            do
            {
                this.m_WaitingForWidgetEvent = true;
                Trace.WriteLine("final waiting...");
                //need to handle application exit event!!!
                int ret = WaitForMultiEvents();
                if (ret == 0)
                {
                    Trace.WriteLine("screen1 quit since " + ret);
                    return false;
                }
                while (eventQ.Count > 0)
                {
                    eventData eobj = (eventData)eventQ.Dequeue();

                    Trace.WriteLine("final got the event:" + eobj.m_wid + ",code =" + eobj.m_ecode);
                    if (eobj.m_wid == bt1.WidgetID && eobj.m_ecode == EventCode.CLICKED)
                    {
                        Trace.WriteLine("button pressed!!!" + eobj.m_ecode);
                        bClicked = true;
                        break;
                    }
                    else
                    {
                        //keep track which button been pressed
                        if (eobj.m_ecode == EventCode.CHECKED)
                        {
                            activerd = eobj.m_wid;
                            Trace.WriteLine("activerd =" + eobj.m_wid);
                        }

                    }
                } //end of while
                Win32.ResetEvent(this.m_hWidgetFeedback);
            } while (bClicked == false);

            if (activerd == rd1.WidgetID)
            {
                bret = true;
            }
            return bret;
        }
        #endregion
        #region screen2
        private bool screen2()
        {
            string text = "   Please check all steps completed  ";
            string text2 = "Please review the agreement and check the appropriate box when ready.";
            string rbtext1 = "I have reviewed the agreement in its entirety.";
            string rbtext2 = "I requested an optional printed copy of the agreement.";
            bool bret = false;
            //remove all the previous widgets
            m_WaitingForWidgetEvent = true;
            vis.Widgets.RemoveAllWidgets();
            int ret = WaitForMultiEvents();
            if (ret == 0)
            {
                return false;
            }
            while (eventQ.Count > 0)
            {
                eventQ.Dequeue();
            }
            Win32.ResetEvent(this.m_hWidgetFeedback);

            WidgetInfo wg16 = new WidgetInfo();
            wg16.WidgetID = 0;
            //make inking disable            
            InkRegion region = vis.SignaturePad.InkRegion;
            region.Enabled = false;
            byte checkbox1 = 0;
            byte checkbox2 = 0;
            vis.SignaturePad.InkRegion = region;

            //make lcd background image off
            vis.SignaturePad.EnableBackgroundImage = false;

            LCDColor blue;
            blue.Red = 0;
            blue.Green = 0;
            blue.Blue = 255;
            blue.Alpha = 255;
            //set the bkcolor same with widget color
            vis.SignaturePad.BackgroundColor = blue;
            //create a fullwidth screen widget text box with text
            WidgetInfo wg1 = new WidgetInfo();
            wg1.WidgetID = 1;
            wg1.BgColor = blue;
            wg1.Text = text;
            wg1.WidgetID = 11;
            wg1.WidgetType = WidgetType.TEXT;
            wg1.FontID = (byte)FontID.SAN_B_24;
            wg1.FgColor.Red = 255;
            wg1.FgColor.Green = 255;
            wg1.FgColor.Blue = 0;
            wg1.FgColor.Alpha = 255;
            wg1.Effect = WidgetEffect.NONE;
            wg1.Left = 10;
            wg1.Top = 10;
            wg1.Width = 478;
            wg1.Height = 40;
            this.m_WaitingForWidgetEvent = true;
            vis.Widgets.CreateWidget(wg1);
            int nRet = WaitForWidgetEvent(wg1.WidgetID, EventCode.CREATE_SUCCEED, EventCode.CREATE_FAILED);
            if (nRet == 2)
            {
                throw new ePadException(HERROR.F_INTERNAL_ERROR);
            }
            Trace.WriteLine("scree2: wg1 creating return " + nRet);

            //create two radio buttons
            WidgetInfo rd1 = new WidgetInfo();
            rd1.BgColor = blue;
            rd1.Text = rbtext1;
            rd1.WidgetID = 13;
            rd1.WidgetType = WidgetType.CHECKBOX;
            rd1.FontID = (byte)FontID.MON_R_18;
            rd1.FgColor.Red = 255;
            rd1.FgColor.Green = 255;
            rd1.FgColor.Blue = 0;
            rd1.FgColor.Alpha = 255;
            rd1.Effect = WidgetEffect.NONE;
            rd1.Left = 20;
            rd1.Top = 60;
            rd1.Width = 400;
            rd1.Height = 20;
            rd1.GroupID = 1;
            this.m_WaitingForWidgetEvent = true;
            vis.Widgets.CreateWidget(rd1);
            nRet = WaitForWidgetEvent(rd1.WidgetID, EventCode.CREATE_SUCCEED, EventCode.CREATE_FAILED);
            if (nRet == 2)
            {
                throw new ePadException(HERROR.F_INTERNAL_ERROR);
            }
            Trace.WriteLine("screen2: rd1 creating return " + nRet + ", 380: length =" + rbtext1.Length);

            WidgetInfo rd2 = new WidgetInfo();

            rd2.BgColor = blue;
            rd2.Text = rbtext2;
            rd2.WidgetID = 14;
            rd2.WidgetType = WidgetType.CHECKBOX;
            rd2.FontID = (byte)FontID.MON_R_18;
            rd2.FgColor.Red = 255;
            rd2.FgColor.Green = 255;
            rd2.FgColor.Blue = 0;
            rd2.FgColor.Alpha = 255;
            rd2.Effect = WidgetEffect.NONE;
            rd2.Left = 20;
            rd2.Top = 100;
            rd2.Width = 455;
            rd2.Height = 20;
            rd2.GroupID = 1;
            this.m_WaitingForWidgetEvent = true;
            vis.Widgets.CreateWidget(rd2);
            nRet = WaitForWidgetEvent(rd2.WidgetID, EventCode.CREATE_SUCCEED, EventCode.CREATE_FAILED);
            if (nRet == 2)
            {
                throw new ePadException(HERROR.F_INTERNAL_ERROR);
            }
            Trace.WriteLine("screen2 rd2 creating return " + nRet + ",100: length =" + rbtext2.Length);

            WidgetInfo bt1 = new WidgetInfo();
            bt1.BgColor.Red = 0;
            bt1.BgColor.Green = 255;
            bt1.BgColor.Blue = 0;
            bt1.BgColor.Alpha = 255;
            bt1.Text = "Proceed";
            bt1.WidgetID = 15;
            bt1.WidgetType = WidgetType.BUTTON;
            bt1.FontID = (byte)FontID.SAN_R_20;
            bt1.FgColor.Red = 0;
            bt1.FgColor.Green = 0;
            bt1.FgColor.Blue = 0;
            bt1.FgColor.Alpha = 255;
            bt1.Effect = WidgetEffect.THREED;
            bt1.Left = 200;
            bt1.Top = 210;
            bt1.Width = 80;
            bt1.Height = 60;
            bt1.GroupID = 2;
            this.m_WaitingForWidgetEvent = true;
            vis.Widgets.CreateWidget(bt1);

            nRet = WaitForWidgetEvent(bt1.WidgetID, EventCode.CREATE_SUCCEED, EventCode.CREATE_FAILED);
            if (nRet == 2)
            {
                throw new ePadException(HERROR.F_INTERNAL_ERROR);
            }
            Trace.WriteLine("screen2 bt1 creating return " + nRet);

            //wait for the events
recheck:
            bool bClicked = false;
            do
            {
                this.m_WaitingForWidgetEvent = true;
                ret = WaitForMultiEvents();
                if (ret == 0)
                {
                    Trace.WriteLine("screen2 quit since " + ret);
                    return false;
                }
                while (eventQ.Count > 0)
                {
                    eventData eobj = (eventData)eventQ.Dequeue();

                    if (eobj.m_wid == bt1.WidgetID && eobj.m_ecode == EventCode.CLICKED)
                    {
                        Trace.WriteLine("screen2: button pressed!!! " + eobj.m_ecode);
                        bClicked = true;
                        break;
                    }
                    else 
                    {
                        //keep track which button been pressed
                        if (eobj.m_wid == rd1.WidgetID )
                        {
                            Trace.WriteLine("screen2 checkbox1 = " + checkbox1);
                            if (wg16.WidgetID != 0)
                            {
                                this.m_WaitingForWidgetEvent = true;
                                vis.Widgets.DeleteWidget(16);
                                if ( WaitForWidgetEvent(16, EventCode.DELETE_DONE) == false ) 
                                {
                                    throw new ePadException(HERROR.F_INTERNAL_ERROR);
                                }
                                Trace.WriteLine("screen2: deleting widget 16...");
                                wg16.WidgetID = 0;
                            }
                            checkbox1 = (byte)eobj.m_ecode;
                            Trace.WriteLine("screen2: id = " + eobj.m_wid + "checkbox1 =" + checkbox1);
                        } else if (eobj.m_wid == rd2.WidgetID)
                        {
                            checkbox2 = (byte)eobj.m_ecode;
                            Trace.WriteLine("screen2: id = " + eobj.m_wid + "checkbox2 =" + checkbox2);
                        }
                    
                    }
                } //end of while
                Win32.ResetEvent(this.m_hWidgetFeedback);
            } while (bClicked == false);

            if (checkbox1 != (byte)EventCode.CHECKED)
            {
                wg16.BgColor = blue;
                wg16.Text = text2;
                wg16.WidgetID = 16;
                wg16.WidgetType = WidgetType.TEXT;
                wg16.FontID = (byte)FontID.MON_R_12;
                //orange
                wg16.FgColor.Red = 255;
                wg16.FgColor.Green = 255;
                wg16.FgColor.Blue = 255;
                wg16.FgColor.Alpha = 255;
                wg16.Effect = WidgetEffect.NONE;
                wg16.Left = 50;
                wg16.Top = 130;
                wg16.Width = 400;
                wg16.Height = 70;
                this.m_WaitingForWidgetEvent = true;
                vis.Widgets.CreateWidget(wg16);
                //this widget may be already cerated when returning Invalid id
                nRet = WaitForWidgetEvent(wg16.WidgetID, EventCode.CREATE_SUCCEED, EventCode.INVALID_ID);
                
                goto recheck;
            } else {
                bret = true;
            }
            return bret;
        }
        #endregion
        #region screen3
        private int screen3()
        {
            int nret = 0;
            //remove all the previous widgets
            m_WaitingForWidgetEvent = true;
            vis.Widgets.RemoveAllWidgets();
            int ret = WaitForMultiEvents();
            if (ret == 0)
            {
                return ret ;
            }
            while (eventQ.Count > 0)
            {
                eventQ.Dequeue();
            }
            Win32.ResetEvent(this.m_hWidgetFeedback);            
            //make inking disable            
            InkRegion region = vis.SignaturePad.InkRegion;
            region.Enabled = false;
         
            byte checkbox1 = 0;
            byte checkbox2 = 0;
            vis.SignaturePad.InkRegion = region;            
            //make lcd background image off
            vis.SignaturePad.EnableBackgroundImage = false;

            LCDColor blue;
            blue.Red = 0;
            blue.Green = 0;
            blue.Blue = 255;
            blue.Alpha = 255;
            LCDColor grey;
            grey.Red = 184;
            grey.Green = 181;
            grey.Blue = 187;
            grey.Alpha = 255;
            //set the bkcolor same with widget color
            vis.SignaturePad.BackgroundColor = blue;

            //create a fullwidth screen widget text box with text
            WidgetInfo wg1 = new WidgetInfo();
            wg1.BgColor = grey;
            wg1.Text = this.CompliText;
            wg1.WidgetID = 1;
            wg1.WidgetType = WidgetType.TEXT;
            wg1.FontID = (byte)FontID.COM_B_24;
            wg1.FgColor.Red = 0;
            wg1.FgColor.Green = 0;
            wg1.FgColor.Blue = 0;
            wg1.FgColor.Alpha = 255;
            wg1.Effect = WidgetEffect.THREED;
            wg1.Left = 10;
            wg1.Top = 10;
            wg1.Width = 460;
            wg1.Height = 180;
            this.m_WaitingForWidgetEvent = true;
            vis.Widgets.CreateWidget(wg1);
            Trace.WriteLine("screen3 cerate widget id = " + wg1.WidgetID);
            bool bwret = WaitForWidgetEvent(wg1.WidgetID, EventCode.CREATE_SUCCEED);

            WidgetInfo bt1 = new WidgetInfo();
            WidgetInfo bt2 = new WidgetInfo();
            bt1.BgColor.Red = 0;
            bt1.BgColor.Green = 255;
            bt1.BgColor.Blue = 0;
            bt1.BgColor.Alpha = 255;
            bt1.Text = "Accept";
            bt1.WidgetID = 2;
            bt1.WidgetType = WidgetType.BUTTON;
            bt1.FontID = (byte)FontID.SAN_B_20;
            bt1.FgColor.Red = 0;
            bt1.FgColor.Green = 0;
            bt1.FgColor.Blue = 0;
            bt1.FgColor.Alpha = 255;
            bt1.Effect = WidgetEffect.THREED;
            bt1.Left = 320;
            bt1.Top = 210;
            bt1.Width = 80;
            bt1.Height = 60;
            bt1.GroupID = 2;
            this.m_WaitingForWidgetEvent = true;
            vis.Widgets.CreateWidget(bt1);
            bwret = WaitForWidgetEvent(bt1.WidgetID, EventCode.CREATE_SUCCEED);
 
            bt2.BgColor.Red = 0;
            bt2.BgColor.Green = 255;
            bt2.BgColor.Blue = 0;
            bt2.BgColor.Alpha = 255;
            bt2.Text = "Decline";
            bt2.WidgetID = 3;
            bt2.WidgetType = WidgetType.BUTTON;
            bt2.FontID = (byte)FontID.SAN_B_20;
            bt2.FgColor.Red = 0;
            bt2.FgColor.Green = 0;
            bt2.FgColor.Blue = 0;
            bt2.FgColor.Alpha = 255;
            bt2.Effect = WidgetEffect.THREED;
            bt2.Left = 80;
            bt2.Top = 210;
            bt2.Width = 80;
            bt2.Height = 60;
            bt2.GroupID = 3;
            this.m_WaitingForWidgetEvent = true;
            vis.Widgets.CreateWidget(bt2);
            bwret = WaitForWidgetEvent(bt2.WidgetID, EventCode.CREATE_SUCCEED);
 
            //wait for the events
            checkbox1 = 0;
            checkbox2 = 0;
            do
            {
                this.m_WaitingForWidgetEvent = true;
                Trace.WriteLine("final waiting...");
                //need to handle application exit event!!!
                ret = WaitForMultiEvents();
                if (ret == 0)
                {
                    Trace.WriteLine("screen1 quit since " + ret);
                    return 0;
                }
                while (eventQ.Count > 0)
                {
                    eventData eobj = (eventData)eventQ.Dequeue();

                    if (eobj.m_wid == bt1.WidgetID)
                    {
                        checkbox1 = (byte)eobj.m_ecode;
                        break;
                    }
                    if (eobj.m_wid == bt2.WidgetID)
                    {
                        checkbox2 = (byte)eobj.m_ecode;
                        break;
                    }
                } //end of while
                Win32.ResetEvent(this.m_hWidgetFeedback);
            } while (checkbox1 == 0 && checkbox2 == 0);
            //accept
            if (checkbox1 == (byte)EventCode.CLICKED)
            {
                nret = 1;
            }
            return nret;
        }
        #endregion
        #region screen4
        private bool screen4()
        {
            bool bret = true;
            //remove all the previous widgets
            m_WaitingForWidgetEvent = true;
            vis.Widgets.RemoveAllWidgets();
            int ret = WaitForMultiEvents();
            if (ret == 0)
            {
                return false;
            }
            while (eventQ.Count > 0)
            {
                eventQ.Dequeue();
            }
            Win32.ResetEvent(this.m_hWidgetFeedback);

           
            InkRegion region = vis.SignaturePad.InkRegion;
            region.Enabled = false;
            byte checkbox1 = 0;
            byte checkbox2 = 0;
            vis.SignaturePad.InkRegion = region;
            Thread.Sleep(500);

            LCDColor grey;
            grey.Red = 184;
            grey.Green = 181;
            grey.Blue = 187;
            grey.Alpha = 255;
            LCDColor blue;
            blue.Red = 0;
            blue.Green = 0;
            blue.Blue = 255;
            blue.Alpha = 255;
            //set the bkcolor same with widget color
            vis.SignaturePad.BackgroundColor = blue;
            //create a fullwidth screen widget text box with text
            WidgetInfo wg1 = new WidgetInfo();
            wg1.BgColor = grey;
            wg1.Text = this.AffirmText;
            wg1.WidgetID = 41;
            wg1.WidgetType = WidgetType.TEXT;
            wg1.FontID = (byte)FontID.SAN_B_24;
            wg1.FgColor.Red = 0;
            wg1.FgColor.Green = 0;
            wg1.FgColor.Blue = 0;
            wg1.FgColor.Alpha = 255;
            wg1.Effect = WidgetEffect.THREED;
            wg1.Left = 10;
            wg1.Top = 10;
            wg1.Width = 460;
            wg1.Height = 50;
            this.m_WaitingForWidgetEvent = true;
            vis.Widgets.CreateWidget(wg1);
            bool bwret = WaitForWidgetEvent(wg1.WidgetID, EventCode.CREATE_SUCCEED);
            

            WidgetInfo bt1 = new WidgetInfo();
            WidgetInfo bt2 = new WidgetInfo();
            WidgetInfo bt3 = new WidgetInfo();
            bt1.BgColor.Red = 0;
            bt1.BgColor.Green = 255;
            bt1.BgColor.Blue = 0;
            bt1.BgColor.Alpha = 255;
            bt1.Text = "Clear";
            bt1.WidgetID = 45;
            bt1.WidgetType = WidgetType.BUTTON;
            bt1.FontID = (byte)FontID.SAN_B_20;
            bt1.FgColor.Red = 0;
            bt1.FgColor.Green = 0;
            bt1.FgColor.Blue = 0;
            bt1.FgColor.Alpha = 255;
            bt1.Effect = WidgetEffect.THREED;
            bt1.Left = 50;
            bt1.Top = 210;
            bt1.Width = 80;
            bt1.Height = 60;
            bt1.GroupID = 2;
            this.m_WaitingForWidgetEvent = true;
            vis.Widgets.CreateWidget(bt1);
            bwret = WaitForWidgetEvent(bt1.WidgetID, EventCode.CREATE_SUCCEED);

            bt2.BgColor.Red = 0;
            bt2.BgColor.Green = 255;
            bt2.BgColor.Blue = 0;
            bt2.BgColor.Alpha = 255;
            bt2.Text = "Cancel";
            bt2.WidgetID = 46;
            bt2.WidgetType = WidgetType.BUTTON;
            bt2.FontID = (byte)FontID.SAN_B_20;
            bt2.FgColor.Red = 0;
            bt2.FgColor.Green = 0;
            bt2.FgColor.Blue = 0;
            bt2.FgColor.Alpha = 255;
            bt2.Effect = WidgetEffect.THREED;
            bt2.Left = 210;
            bt2.Top = 210;
            bt2.Width = 80;
            bt2.Height = 60;
            bt2.GroupID = 3;
            this.m_WaitingForWidgetEvent = true;
            vis.Widgets.CreateWidget(bt2);
            bwret = WaitForWidgetEvent(bt2.WidgetID, EventCode.CREATE_SUCCEED);

            bt3.BgColor.Red = 0;
            bt3.BgColor.Green = 255;
            bt3.BgColor.Blue = 0;
            bt3.BgColor.Alpha = 255;
            bt3.Text = "OK";
            bt3.WidgetID = 47;
            bt3.WidgetType = WidgetType.BUTTON;
            bt3.FontID = (byte)FontID.SAN_B_20;
            bt3.FgColor.Red = 0;
            bt3.FgColor.Green = 0;
            bt3.FgColor.Blue = 0;
            bt3.FgColor.Alpha = 255;
            bt3.Effect = WidgetEffect.THREED;
            bt3.Left = 370;
            bt3.Top = 210;
            bt3.Width = 80;
            bt3.Height = 60;
            bt3.GroupID = 4;
            this.m_WaitingForWidgetEvent = true;
            vis.Widgets.CreateWidget(bt3);
            bwret = WaitForWidgetEvent(bt3.WidgetID, EventCode.CREATE_SUCCEED);


            vis.SignaturePad.EnableCustomizedBackgroundImage = true;

            //signature pad area
            region.Left = 0;
            region.Top = 60;
            region.Width = 480;
            region.Height = 150;
            region.Enabled = true;
            region.PenColor.Red = 0;
            region.PenColor.Alpha = 255;
            region.PenColor.Blue = 0;
            region.PenColor.Green = 0;
            region.PenWidth = 3;
            region.Transform = false;
            region.ShowInkTrails = true;
            region.InkTrailsTimeout = 0;
            vis.SignaturePad.InkRegion = region;
            //wait for the events
            do
            {
                this.m_WaitingForWidgetEvent = true;
                Trace.WriteLine("final waiting...");
                //need to handle application exit event!!!
                ret = WaitForMultiEvents();
                if (ret == 0)
                {
                    Trace.WriteLine("screen4.2 quit since " + ret);
                    return false;
                } 
                while ( eventQ.Count > 0 ) {
                    eventData eobj = (eventData)eventQ.Dequeue();   
                    if (eobj.m_wid == bt3.WidgetID && eobj.m_ecode == EventCode.CLICKED)
                    {
                        Trace.WriteLine("button pressed!!!" + eobj.m_ecode);
                        checkbox1 = (byte)eobj.m_ecode;
                        break;
                    }
                    else
                    {
                        //keep track which button been pressed
                        if (eobj.m_wid == bt1.WidgetID)
                        {
                            //clear
                            vis.SignaturePad.Refresh();
                        }
                        if (eobj.m_wid == bt2.WidgetID)
                        {
                            //cancel
                            checkbox2 = (byte)eobj.m_ecode;
                            Trace.WriteLine("id = " + eobj.m_wid + "status =" + checkbox2);
                            break;
                        }
                    }
                } // end of while
                Win32.ResetEvent(this.m_hWidgetFeedback);
            } while (checkbox1 == 0 && checkbox2 == 0);

            //cancel
            if (checkbox2 == (byte)EventCode.CLICKED)
            {
                bret = false;
            }
            vis.SignaturePad.EnableCustomizedBackgroundImage = false;
            return bret;
        }
        #endregion
        #region screen5
        private void screen5(int useText)
        {
            string text = "Thank you for your business. Your transaction is complete.";
            string text2 = "Thank you for your patronage. The transaction has been cancelled. Please see a customer service representative for further assistance.";
            //bool bret = false;
            //remove all the previous widgets
            m_WaitingForWidgetEvent = true;
            vis.Widgets.RemoveAllWidgets();
            int ret = WaitForMultiEvents();
            if (ret == 0)
            {
                return;
            }
            while (eventQ.Count > 0)
            {
                eventQ.Dequeue();
            }
            Win32.ResetEvent(this.m_hWidgetFeedback);
            //make inking disable            
            InkRegion region = vis.SignaturePad.InkRegion;
            region.Enabled = false;
            vis.SignaturePad.InkRegion = region;
            Thread.Sleep(500);
            //make lcd background image off
            vis.SignaturePad.EnableBackgroundImage = false;

            LCDColor blue;
            blue.Red = 0;
            blue.Green = 0;
            blue.Blue = 255;
            blue.Alpha = 255;
            //set the bkcolor same with widget color
            vis.SignaturePad.BackgroundColor = blue;
            //create a fullwidth screen widget text box with text
            WidgetInfo wg1 = new WidgetInfo();
            wg1.BgColor = blue;
            if (useText == 0)
                wg1.Text = text2;
            else
                wg1.Text = text;

            wg1.WidgetID = 1;
            wg1.WidgetType = WidgetType.TEXT;
            wg1.FontID = (byte)FontID.SAN_B_24;
            wg1.FgColor.Red = 255;
            wg1.FgColor.Green = 255;
            wg1.FgColor.Blue = 0;
            wg1.FgColor.Alpha = 255;
            wg1.Effect = WidgetEffect.NONE;
            wg1.Left = 10;
            wg1.Top = 10;
            wg1.Width = 478;
            wg1.Height = 200;
            this.m_WaitingForWidgetEvent = true;
            vis.Widgets.CreateWidget(wg1);
            bool bwret = WaitForWidgetEvent(wg1.WidgetID, EventCode.CREATE_SUCCEED);

            WidgetInfo bt1 = new WidgetInfo();
            bt1.BgColor.Red = 0;
            bt1.BgColor.Green = 255;
            bt1.BgColor.Blue = 0;
            bt1.BgColor.Alpha = 255;
            bt1.Text = "OK";
            bt1.WidgetID = 2;
            bt1.WidgetType = WidgetType.BUTTON;
            bt1.FontID = (byte)FontID.SAN_B_20;
            bt1.FgColor.Red = 0;
            bt1.FgColor.Green = 0;
            bt1.FgColor.Blue = 0;
            bt1.FgColor.Alpha = 255;
            bt1.Effect = WidgetEffect.THREED;
            bt1.Left = 200;
            bt1.Top = 210;
            bt1.Width = 80;
            bt1.Height = 60;
            bt1.GroupID = 2;
            this.m_WaitingForWidgetEvent = true;
            vis.Widgets.CreateWidget(bt1);
            bwret = WaitForWidgetEvent(bt1.WidgetID, EventCode.CREATE_SUCCEED);

            //wait for the events
            bool bChecked = false;
            do
            {
                this.m_WaitingForWidgetEvent = true;
                Trace.WriteLine("final waiting...");
                //need to handle application exit event!!!
                ret = WaitForMultiEvents();
                if (ret == 0)
                {
                    Trace.WriteLine("screen5 quit since " + ret);
                    return;
                }
                while (eventQ.Count > 0)
                {
                    eventData eobj = (eventData)eventQ.Dequeue();
                    Trace.WriteLine("final got the event:" + eobj.m_wid + ",code =" + eobj.m_ecode);
                    if (eobj.m_wid == bt1.WidgetID && eobj.m_ecode == EventCode.CLICKED)
                    {
                        Trace.WriteLine("button pressed!!!" + eobj.m_ecode);
                        bChecked = true;
                        break;
                    }
                } // end of while
                Win32.ResetEvent(this.m_hWidgetFeedback);
            } while (bChecked != true);

            //done with demo
            this.m_WaitingForWidgetEvent = true;
            vis.Widgets.RemoveAllWidgets();
            ret = WaitForMultiEvents();
            if (ret == 0)
            {
                return;
            }
            while (eventQ.Count > 0)
            {
                Trace.WriteLine("deleting eventQ count=" + eventQ.Count);
                eventQ.Dequeue();
            }
            Win32.ResetEvent(this.m_hWidgetFeedback);
            return;
        }
        #endregion
    }
}
